#ifndef _BIOLOID_REGISTERS_H
#define _BIOLOID_REGISTERS_H

#ifdef __cplusplus
extern "C" {
#endif

#define DEVICE_MODEL_OFFSET      ((unsigned short int)0x0000)
#define FIRMWARE_VERSION_OFFSET  ((unsigned short int)0x0002)
#define DEVICE_ID_OFFSET         ((unsigned short int)0x0003)
#define BAUDRATE_OFFSET          ((unsigned short int)0x0004)
#define RETURN_DELAY_OFFSET      ((unsigned short int)0x0005)
#define MM_PERIOD_OFFSET         ((unsigned short int)0x0006)
#define GYRO_FB_ADC_CH_OFFSET    ((unsigned short int)0x0007)
#define GYRO_LR_ADC_CH_OFFSET    ((unsigned short int)0x0008)
#define RETURN_LEVEL_OFFSET      ((unsigned short int)0x0010)

#define LAST_EEPROM_OFFSET       ((unsigned short int)0x0020)

typedef enum {
  BIOLOID_MODEL_NUMBER_L          = DEVICE_MODEL_OFFSET,
  BIOLOID_MODEL_NUMBER_H          = DEVICE_MODEL_OFFSET+1,
  BIOLOID_VERSION                 = FIRMWARE_VERSION_OFFSET,
  BIOLOID_ID                      = DEVICE_ID_OFFSET,
  BIOLOID_BAUD_RATE               = BAUDRATE_OFFSET,
  BIOLOID_RETURN_DELAY_TIME       = RETURN_DELAY_OFFSET,
  BIOLOID_MM_PERIOD_L             = MM_PERIOD_OFFSET,
  BIOLOID_MM_PERIOD_H             = MM_PERIOD_OFFSET+1,
  BIOLIOD_GYRO_FB_ADC_CH          = GYRO_FB_ADC_CH_OFFSET,
  BIOLIOD_GYRO_LR_ADC_CH          = GYRO_LR_ADC_CH_OFFSET,
  BIOLOID_RETURN_LEVEL            = RETURN_LEVEL_OFFSET,
  BIOLOID_USER1_LED_CNTRL         = 0x20, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2  | bit 1 |      bit 0
                                          //       |       |       |       | blink | toggle | value | internally used
  BIOLOID_USER1_LED_PERIOD_L      = 0x21,
  BIOLOID_USER1_LED_PERIOD_H      = 0x22,     
  BIOLOID_USER2_LED_CNTRL         = 0x23, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2  | bit 1 |      bit 0
                                          //       |       |       |       | blink | toggle | value | internally used
  BIOLOID_USER2_LED_PERIOD_L      = 0x24,
  BIOLOID_USER2_LED_PERIOD_H      = 0x25,     
  BIOLOID_RXD_LED_CNTRL           = 0x26, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2  | bit 1 |      bit 0
                                          //       |       |       |       | blink | toggle | value | internally used
  BIOLOID_RXD_LED_PERIOD_L        = 0x27,
  BIOLOID_RXD_LED_PERIOD_H        = 0x28,     
  BIOLOID_TXD_LED_CNTRL           = 0x29, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2  | bit 1 |      bit 0
                                          //       |       |       |       | blink | toggle | value | internally used
  BIOLOID_TXD_LED_PERIOD_L        = 0x2A,
  BIOLOID_TXD_LED_PERIOD_H        = 0x2B,     
  BIOLOID_USER_PB_CNTRL           = 0x2C, // bit 7 | bit 6 | bit 5 | bit 4 |     bit 3      |       bit 2      | bit 1 |      bit 0
                                          //       |       |       |       | interrupt flag | enable interrupt | value | internally used
  BIOLOID_RESET_PB_CNTRL          = 0x2D, // bit 7 | bit 6 | bit 5 | bit 4 |     bit 3      |       bit 2      | bit 1 |      bit 0
                                          //       |       |       |       | interrupt flag | enable interrupt | value | internally used
  BIOLOID_START_PB_CNTRL          = 0x2E, // bit 7 | bit 6 | bit 5 | bit 4 |     bit 3      |       bit 2      | bit 1 |      bit 0
                                          //       |       |       |       | interrupt flag | enable interrupt | value | internally used
  BIOLOID_MODE_PB_CNTRL           = 0x2F, // bit 7 | bit 6 | bit 5 | bit 4 |     bit 3      |       bit 2      | bit 1 |      bit 0
                                          //       |       |       |       | interrupt flag | enable interrupt | value | internally used
  BIOLOID_ADC_CNTRL               = 0x30, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2 | bit 1 | bit 0
                                          //       |       |       |       |       |       |       | Enable
  BIOLOID_ADC_PERIOD              = 0x31,
  BIOLOID_ADC_CH1_L               = 0x32,
  BIOLOID_ADC_CH1_H               = 0x33,
  BIOLOID_ADC_CH2_L               = 0x34,
  BIOLOID_ADC_CH2_H               = 0x35,
  BIOLOID_ADC_CH3_L               = 0x36,
  BIOLOID_ADC_CH3_H               = 0x37,
  BIOLOID_ADC_CH4_L               = 0x38,
  BIOLOID_ADC_CH4_H               = 0x39,
  BIOLOID_ADC_TEMP_L              = 0x3A,
  BIOLOID_ADC_TEMP_H              = 0x3B,
  BIOLOID_ADC_CH6_L               = 0x3C,
  BIOLOID_ADC_CH6_H               = 0x3D,
  BIOLOID_ADC_VREF_L              = 0x3E,
  BIOLOID_ADC_VREF_H              = 0x3F,
  BIOLOID_ADC_CH8_L               = 0x40,
  BIOLOID_ADC_CH8_H               = 0x41,
  BIOLOID_ZIGBEE_CNTRL            = 0x42, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 | bit 2 |  bit 1  |    bit 0
                                          //       |       |       |       |       |       | Enable  | Enable power
  BIOLOID_ZIGBEE_BAUDRATE         = 0x43,
  BIOLOID_ZIGBEE_OWN_ID_L         = 0x44,
  BIOLOID_ZIGBEE_OWN_ID_H         = 0x45,
  BIOLOID_ZIGBEE_REM_ID_L         = 0x46,
  BIOLOID_ZIGBEE_REM_ID_H         = 0x47,
  BIOLOID_MM_NUM_SERVOS           = 0x48,
  BIOLOID_MM_CNTRL                = 0x49, // bit 7 | bit 6 | bit 5 | bit 4 | bit 3 |     bit 2    |      bit 1      |    bit 0
                                          //       |       |       |       |       | Enable power | Enable balance  | Enable manager
  BIOLOID_MM_MODULE_EN0           = 0x4A, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0i
                                          // Enable servo 0  |   assigned module     | Enable servo 1  |   assigned module
                                          //                 |     000 -> none       |
                                          //                 |     001 -> action     |
                                          //                 |     010 -> walk       |
                                          //                 |     011 -> joints     |
  BIOLOID_MM_MODULE_EN1           = 0x4B, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 2  |   assigned module     | Enable servo 3  |   assigned module
  BIOLOID_MM_MODULE_EN2           = 0x4C, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 4  |   assigned module     | Enable servo 5  |   assigned module
  BIOLOID_MM_MODULE_EN3           = 0x4D, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 6  |   assigned module     | Enable servo 7  |   assigned module
  BIOLOID_MM_MODULE_EN4           = 0x4E, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 8  |   assigned module     | Enable servo 9  |   assigned module
  BIOLOID_MM_MODULE_EN5           = 0x4F, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 10 |   assigned module     | Enable servo 11 |   assigned module
  BIOLOID_MM_MODULE_EN6           = 0x50, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 12 |   assigned module     | Enable servo 13 |   assigned module
  BIOLOID_MM_MODULE_EN7           = 0x51, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 14 |   assigned module     | Enable servo 15 |   assigned module
  BIOLOID_MM_MODULE_EN8           = 0x52, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 16 |   assigned module     | Enable servo 17 |   assigned module
  BIOLOID_MM_MODULE_EN9           = 0x53, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 18 |   assigned module     | Enable servo 19 |   assigned module
  BIOLOID_MM_MODULE_EN10          = 0x54, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 20 |   assigned module     | Enable servo 21 |   assigned module
  BIOLOID_MM_MODULE_EN11          = 0x55, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 22 |   assigned module     | Enable servo 23 |   assigned module
  BIOLOID_MM_MODULE_EN12          = 0x56, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 24 |   assigned module     | Enable servo 25 |   assigned module
  BIOLOID_MM_MODULE_EN13          = 0x57, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 26 |   assigned module     | Enable servo 27 |   assigned module
  BIOLOID_MM_MODULE_EN14          = 0x58, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 28 |   assigned module     | Enable servo 29 |   assigned module
  BIOLOID_MM_MODULE_EN15          = 0x59, //      bit 7      | bit 6 | bit 5 | bit 4 |      bit 3      | bit 2 | bit 1 | bit 0
                                          // Enable servo 30 |   assigned module     | Enable servo 31 |   assigned module
  BIOLOID_MM_SERVO0_CUR_POS_L     = 0x5A,
  BIOLOID_MM_SERVO0_CUR_POS_H     = 0x5B,
  BIOLOID_MM_SERVO1_CUR_POS_L     = 0x5C,
  BIOLOID_MM_SERVO1_CUR_POS_H     = 0x5D,
  BIOLOID_MM_SERVO2_CUR_POS_L     = 0x5E,
  BIOLOID_MM_SERVO2_CUR_POS_H     = 0x5F,
  BIOLOID_MM_SERVO3_CUR_POS_L     = 0x60,
  BIOLOID_MM_SERVO3_CUR_POS_H     = 0x61,
  BIOLOID_MM_SERVO4_CUR_POS_L     = 0x62,
  BIOLOID_MM_SERVO4_CUR_POS_H     = 0x63,
  BIOLOID_MM_SERVO5_CUR_POS_L     = 0x64,
  BIOLOID_MM_SERVO5_CUR_POS_H     = 0x65,
  BIOLOID_MM_SERVO6_CUR_POS_L     = 0x66,
  BIOLOID_MM_SERVO6_CUR_POS_H     = 0x67,
  BIOLOID_MM_SERVO7_CUR_POS_L     = 0x68,
  BIOLOID_MM_SERVO7_CUR_POS_H     = 0x69,
  BIOLOID_MM_SERVO8_CUR_POS_L     = 0x6A,
  BIOLOID_MM_SERVO8_CUR_POS_H     = 0x6B,
  BIOLOID_MM_SERVO9_CUR_POS_L     = 0x6C,
  BIOLOID_MM_SERVO9_CUR_POS_H     = 0x6D,
  BIOLOID_MM_SERVO10_CUR_POS_L    = 0x6E,
  BIOLOID_MM_SERVO10_CUR_POS_H    = 0x6F,
  BIOLOID_MM_SERVO11_CUR_POS_L    = 0x70,
  BIOLOID_MM_SERVO11_CUR_POS_H    = 0x71,
  BIOLOID_MM_SERVO12_CUR_POS_L    = 0x72,
  BIOLOID_MM_SERVO12_CUR_POS_H    = 0x73,
  BIOLOID_MM_SERVO13_CUR_POS_L    = 0x74,
  BIOLOID_MM_SERVO13_CUR_POS_H    = 0x75,
  BIOLOID_MM_SERVO14_CUR_POS_L    = 0x76,
  BIOLOID_MM_SERVO14_CUR_POS_H    = 0x77,
  BIOLOID_MM_SERVO15_CUR_POS_L    = 0x78,
  BIOLOID_MM_SERVO15_CUR_POS_H    = 0x79,
  BIOLOID_MM_SERVO16_CUR_POS_L    = 0x7A,
  BIOLOID_MM_SERVO16_CUR_POS_H    = 0x7B,
  BIOLOID_MM_SERVO17_CUR_POS_L    = 0x7C,
  BIOLOID_MM_SERVO17_CUR_POS_H    = 0x7D,
  BIOLOID_MM_SERVO18_CUR_POS_L    = 0x7E,
  BIOLOID_MM_SERVO18_CUR_POS_H    = 0x7F,
  BIOLOID_MM_SERVO19_CUR_POS_L    = 0x80,
  BIOLOID_MM_SERVO19_CUR_POS_H    = 0x81,
  BIOLOID_MM_SERVO20_CUR_POS_L    = 0x82,
  BIOLOID_MM_SERVO20_CUR_POS_H    = 0x83,
  BIOLOID_MM_SERVO21_CUR_POS_L    = 0x84,
  BIOLOID_MM_SERVO21_CUR_POS_H    = 0x85,
  BIOLOID_MM_SERVO22_CUR_POS_L    = 0x86,
  BIOLOID_MM_SERVO22_CUR_POS_H    = 0x87,
  BIOLOID_MM_SERVO23_CUR_POS_L    = 0x88,
  BIOLOID_MM_SERVO23_CUR_POS_H    = 0x89,
  BIOLOID_MM_SERVO24_CUR_POS_L    = 0x8A,
  BIOLOID_MM_SERVO24_CUR_POS_H    = 0x8B,
  BIOLOID_MM_SERVO25_CUR_POS_L    = 0x8C,
  BIOLOID_MM_SERVO25_CUR_POS_H    = 0x8D,
  BIOLOID_MM_SERVO26_CUR_POS_L    = 0x8E,
  BIOLOID_MM_SERVO26_CUR_POS_H    = 0x8F,
  BIOLOID_MM_SERVO27_CUR_POS_L    = 0x90,
  BIOLOID_MM_SERVO27_CUR_POS_H    = 0x91,
  BIOLOID_MM_SERVO28_CUR_POS_L    = 0x92,
  BIOLOID_MM_SERVO28_CUR_POS_H    = 0x93,
  BIOLOID_MM_SERVO29_CUR_POS_L    = 0x94,
  BIOLOID_MM_SERVO29_CUR_POS_H    = 0x95,
  BIOLOID_MM_SERVO30_CUR_POS_L    = 0x96,
  BIOLOID_MM_SERVO30_CUR_POS_H    = 0x97,
  BIOLOID_MM_SERVO31_CUR_POS_L    = 0x98,
  BIOLOID_MM_SERVO31_CUR_POS_H    = 0x99,
  BIOLOID_ACTION_PAGE             = 0x9A,
  BIOLOID_ACTION_CNTRL            = 0x9B, // bit 7 | bit 6 | bit 5 |     bit 4    |     bit 3      |       bit 2      |   bit 1   |   bit 0
                                          //       |       |       | page running | interrupt flag | enable interrupt | stop page | start page
  BIOLOID_GYRO_CNTRL              = 0x9C, // bit 7 | bit 6 | bit 5 |     bit 4    |     bit 3      |       bit 2      |   bit 1   |   bit 0
                                          // 
  BIOLOID_GYRO_STATUS             = 0x9D, // bit 7 | bit 6 | bit 5 |     bit 4    |     bit 3      |       bit 2      |   bit 1   |   bit 0
                                          // 
  BIOLOID_GYRO_FWD_FALL_THD       = 0x9E,
  BIOLOID_GYRO_BWD_FALL_THD       = 0x9F,
  BIOLOID_GYRO_LEFT_FALL_THD      = 0xA0,
  BOILOID_GYRO_RIGHT_FALL_THD     = 0xA1
} bioloid_registers;

#define      GPIO_BASE_ADDRESS    0x20
#define      GPIO_MEM_LENGTH      16
#define      GPIO_INT_USED        0x01
#define      GPIO_VALUE           0x02
#define      GPIO_TOGGLE          0x04
#define      GPIO_BLINK           0x08
#define      GPIO_INT_EN          0x02
#define      GPIO_INT_FLAG        0x04

#define      ADC_BASE_ADDRESS     0x30
#define      ADC_MEM_LENGTH       18
#define      ADC_ENABLE           0x01

#define      ZIGBEE_BASE_ADDRESS  0x42
#define      ZIGBEE_MEM_LENGTH    6
#define      ZIGBEE_EN_PWR        0x01
#define      ZIGBEE_ENABLE        0x02

#define      MANAGER_BASE_ADDRESS 0x48
#define      MANAGER_MEM_LENGTH   50
#define      MANAGER_ENABLE       0x01
#define      MANAGER_EN_BAL       0x02
#define      MANAGER_EN_PWR       0x04
#define      MANAGER_EVEN_SER_EN  0x80
#define      MANAGER_EVEN_SER_MOD 0x70
#define      MANAGER_ODD_SER_EN   0x08
#define      MANAGER_ODD_SER_MOD  0x07

#define      ACTION_BASE_ADDRESS  0x9A
#define      ACTION_MEM_LENGTH    2
#define      ACTION_START         0x01
#define      ACTION_STOP          0x02
#define      ACTION_INT_EN        0x04
#define      ACTION_INT_FLAG      0x08
#define      ACTION_STATUS        0x10

#ifdef __cplusplus
}
#endif

#endif
